#include <stdio.h>
#include "instructions.h"
#include "instructionParameters.h"
#include "matrixPointers.h"
#include "cLib.h"

#define createSec(m, i, j, x) \
            (m + i*BlockSize*Size*4 + (FU_Number*j+x)*BlockSize*4)

#define createPrim(m, i, j) \
            (m + i*BlockSize*Size*4 + j*BlockSize*4)

void goodAlgo()
{
    int Size = 64;
    int BlockSize = 64;
    UInt64 a = (UInt64)aMatrix;
    UInt64 b = (UInt64)bMatrix;
    UInt64 c = (UInt64)cMatrix;

    int FU_Number = (int) FunctionalUnitNumber;

    int BlockNum = Size/BlockSize;
    int LogSize;

    int BigBlockNum = BlockNum/FU_Number;
    int BigBlockRest = BlockNum%FU_Number;

    int response;

    switch(Size)
    {
        case 64:
            LogSize = 6;
            break;
        case 128:
            LogSize = 7;
            break;
        case 256:
            LogSize = 8;
            break;
        case 512:
            LogSize = 9;
            break;
        default:
            LogSize = 10;
            break;
    }

    initialize();
    putInstruction(createSetRowSizeInstruction(LogSize));

    int i;
    int j;
    int k;
    int f;

    for(i = 0; i < BlockNum; i=i+1)
    {
        for(j = 0; j < BigBlockNum; j=j+1)
        {
            putInstruction(createArithmeticInstruction(All_FU_Mask, Zero));
            for(k = 0; k < BlockNum; k=k+1)
            {
                for(f = 0; f < FU_Number; f=f+1)
                {
                    putInstruction(createLoadInstruction((((int)1)<<f), B, createSec(b, k, j, f)));
                }
                putInstruction(createLoadInstruction(All_FU_Mask, A, createPrim(a, i, k)));
                putInstruction(createArithmeticInstruction(All_FU_Mask, MultiplyAddAccumulate));
            }
            for(f = 0; f < FU_Number; f=f+1)
            {
                putInstruction(createStoreInstruction(f, C, createSec(c, i, j, f)));
            }
        }
        if(BigBlockRest != 0)
        {
            putInstruction(createArithmeticInstruction(All_FU_Mask, Zero));
            for(k = 0; k < BlockNum; k=k+1)
            {
                for(f = 0; f < BigBlockRest; f=f+1)
                {
                    putInstruction(createLoadInstruction(((int)1<<f), B, createPrim(b, k, j*FU_Number+f)));
                }
                putInstruction(createLoadInstruction(All_FU_Mask, A, createPrim(a, i, k)));
                putInstruction(createArithmeticInstruction(All_FU_Mask, MultiplyAddAccumulate));
            }
            for(f = 0; f < BigBlockRest; f=f+1)
            {
                putInstruction(createStoreInstruction(f, C, createPrim(c, i, j*FU_Number+f)));
            }
        }
    }

    response = getResponse();
    if(response == (Size/BlockSize)*(Size/BlockSize))
        printf("Passed\n");
    else if(response != 0)
        printf("Failed\n");
    else
        fprintf(stderr, "Passed if simulation\n");
}

int main()
{
    goodAlgo();
}
