module multiplyAdder
(
    CLK,
    CLK_GATE,
    x,
    y,
    accum,
    is_load,
    mulAdd,
    enable
);

    input CLK;
    input CLK_GATE;
    input [31:0] x;
    input [31:0] y;
    input  is_load;
    input [31:0] accum;
    output [31:0] mulAdd;

    input enable;

    wire [15:0] a = x[31:16];
    wire [15:0] b = x[15:0];
    wire [15:0] c = y[31:16];
    wire [15:0] d = y[15:0];

    reg [15:0] ac;
    reg [15:0] bd;
    reg [15:0] ad;
    reg [15:0] bc;

    wire [15:0] ac_wire;
    wire [15:0] bd_wire;
    wire [15:0] ad_wire;
    wire [15:0] bc_wire;

    /*
    mult mul1(.mult_fp_1(a), .mult_fp_2(c), .mult_fp(ac_wire));
    mult mul2(.mult_fp_1(b), .mult_fp_2(d), .mult_fp(bd_wire));
    mult mul3(.mult_fp_1(a), .mult_fp_2(d), .mult_fp(ad_wire));
    mult mul4(.mult_fp_1(b), .mult_fp_2(c), .mult_fp(bc_wire));
    */

    always@(posedge CLK)
    begin
        if(enable)
        begin
            ac <= ac_wire;
            bd <= bd_wire;
            ad <= ad_wire;
            bc <= bc_wire;
        end
    end

    reg [31:0] accum_reg1;

    always@(posedge CLK)
    begin
        if(enable)
        begin
            if(is_load)
                accum_reg1 <= accum;
            else
                accum_reg1 <= mulAdd;
        end
    end

    wire [15:0] r_part = ac-bd;
    wire [15:0] i_part = ad+bc;

    wire [15:0] real_part = r_part+accum_reg1[31:16];
    wire [15:0] imag_part = i_part+accum_reg1[15:0];

    assign mulAdd = {real_part, imag_part};

endmodule
