// Global imports
import Connectable::*;
import Vector::*;

// Local imports
`include "Common.bsv"

import PLBMasterWires::*;
import BRAMTargetWires::*;
import BRAMInitiatorWires::*;
import SimpleMemorySwitch::*;
import PLBMaster::*;
import SimpleFUNetwork::*;
import SimpleController::*;
import FunctionalUnit::*;
//import PPCFeeder::*;
import BRAMFeeder::*;
import BRAMModel::*;


/* This is the top level interface. We only have wires going to the 
   plb bus and the DSOCM which implements the BRAM interface. 
*/
(*synthesize*)
module mkMultiplierTop (Multiplier);
   
  
  Feeder feeder <- mkBRAMFeeder();

  PLBMaster     plbMaster <- mkPLBMaster;
  MemorySwitch  memorySwitch <- mkSimpleMemorySwitch();
  Vector#(FunctionalUnitNumber, FunctionalUnit#(8)) fus <- replicateM(mkFunctionalUnit_STRIPPED8);
  




  let fu_links = map(getLink, fus);
  
  FUNetwork     funet <- mkSimpleNetwork(fu_links);
  Controller    controller <- mkSimpleController();

  mkConnection(feeder.ppcInstructionOutput, controller.instructionInput);
  mkConnection(plbMaster.wordInput, memorySwitch.plbMasterComplexWordOutput);
  mkConnection(plbMaster.wordOutput, memorySwitch.plbMasterComplexWordInput);
  mkConnection(controller.plbMasterCommandOutput, plbMaster.plbMasterCommandInput);
  mkConnection(controller.fuNetworkCommandOutput, funet.fuNetworkCommandInput);
  mkConnection(controller.memorySwitchCommandOutput, memorySwitch.memorySwitchCommandInput);

  mkConnection(memorySwitch.getInstructionOutput, feeder.ppcMessageInput);
  
  for (Integer x = 0; x < valueof(FunctionalUnitNumber); x = x + 1)
  begin
    mkConnection(controller.functionalUnitCommandOutputs[x], fus[x].functionalUnitCommandInput);
    mkConnection(memorySwitch.functionalUnitComplexWordOutputs[x], fus[x].switchInput);
    mkConnection(fus[x].switchOutput, memorySwitch.functionalUnitComplexWordInputs[x]);
  end
  
  interface plbMasterWires = plbMaster.plbMasterWires;
  //interface bramTargetWires = feeder.bramTargetWires;
  interface bramInitiatorWires = feeder.bramInitiatorWires;  

endmodule
