// Global imports
import GetPut::*;
import FIFO::*;
import Vector::*;

// Local imports 
`include "Common.bsv"

(* synthesize *) 
module mkSimpleMemorySwitch (MemorySwitch);

  FIFO#(ComplexWord)             plbMasterComplexWordInfifo <- mkFIFO();
  FIFO#(ComplexWord)             plbMasterComplexWordOutfifo <- mkFIFO();

  FIFO#(PPCMessage)              ppcMessagefifo <- mkFIFO();
  Reg#(PPCMessage)               ppcMessageVal  <- mkReg(1);

  FIFO#(MemorySwitchCommand)     memorySwitchCommandInfifo <- mkFIFO();
  Vector#(FunctionalUnitNumber,
          FIFO#(ComplexWord))    functionalUnitComplexWordOutfifos <- replicateM(mkFIFO());
  Vector#(FunctionalUnitNumber,
          FIFO#(ComplexWord))    functionalUnitComplexWordInfifos <- replicateM(mkFIFO());
  
  Reg#(Bit#(LogBlockElements))   elementCounter <- mkReg(0);
  
  rule routeStore(memorySwitchCommandInfifo.first() matches tagged StoreFromFU .fu);
    // Since the block size is always a power of two, Overflow on the counter means 
    // we deque the command as we have finished
    if(elementCounter == 0)
      begin
        debug(memorySwitchDebug, $display("memorySwitch: processing StoreFromFU"));
      end

    if(elementCounter + 1 == 0)
      begin
        debug(memorySwitchDebug, $display("memorySwitch: finished processing StoreFromFU"));
        ppcMessagefifo.enq(ppcMessageVal);
        ppcMessageVal <= ppcMessageVal+1;
        memorySwitchCommandInfifo.deq();
      end
    elementCounter <= elementCounter + 1;
    plbMasterComplexWordOutfifo.enq(
    functionalUnitComplexWordInfifos[fu].first());
    functionalUnitComplexWordInfifos[fu].deq();
  endrule

  rule routeLoad(memorySwitchCommandInfifo.first() matches tagged LoadToFUs .fus);
    // Since the block size is always a power of two, Overflow on the counter means
    // we deque the command as we have finished
    if(elementCounter == 0)
      begin
        debug(memorySwitchDebug, $display("memorySwitch: processing LoadToFUs"));
      end

    if(elementCounter + 1 == 0)
      begin
        debug(memorySwitchDebug, $display("memorySwitch: finished processing LoadToFUs"));
        memorySwitchCommandInfifo.deq();
      end
    elementCounter <= elementCounter + 1;
    for (Integer x = 0; x < valueof(FunctionalUnitNumber); x = x + 1)
      if (fus[x] == 1)
        functionalUnitComplexWordOutfifos[x].enq(plbMasterComplexWordInfifo.first());
    //$display("SWITCH: %h", plbMasterComplexWordInfifo.first());
    plbMasterComplexWordInfifo.deq();
  endrule


  interface Put plbMasterComplexWordInput = fifoToPut(plbMasterComplexWordInfifo) ;
  interface Get plbMasterComplexWordOutput = fifoToGet(plbMasterComplexWordOutfifo);
  interface Put memorySwitchCommandInput = fifoToPut(memorySwitchCommandInfifo);
  interface functionalUnitComplexWordOutputs = map(fifoToGet,functionalUnitComplexWordOutfifos); 
  interface functionalUnitComplexWordInputs = map(fifoToPut,functionalUnitComplexWordInfifos); 
  interface Get getInstructionOutput = fifoToGet(ppcMessagefifo);

endmodule
